module QuadPres where

import Url
import Contents_Abs
import Contents


navigation_style_class = "nav"
contents_style_class = "contents"

data QuadPresCoords coords doc_type mime =
    MakeQuadPresCoords coords doc_type mime

get_coords_coords (MakeQuadPresCoords coords doc_type mime) = coords
get_coords_doc_type (MakeQuadPresCoords coords doc_type mime) = doc_type
get_coords_mime (MakeQuadPresCoords coords doc_type mime) = mime

data QuadPresCoordsDocType = Image | Node

instance Eq QuadPresCoordsDocType where
    Image == Image = True
    Node == Node = True
    a == b = False

instance Show QuadPresCoordsDocType where
    show Image = "Image"
    show Node = "Node"


type QuadPresCoordsType = QuadPresCoords [Int] QuadPresCoordsDocType String

instance (Show a, Show b, Show c) => Show (QuadPresCoords a b c) where
    show (MakeQuadPresCoords coords doc_type mime) =
        "QuadPresCoords (coords = " ++ (show coords) ++ ", " ++
        "doc_type = " ++ (show doc_type) ++ ", " ++
        "mime = " ++ (show mime) ++ ")"

data QuadPres contents doc_id mode
    doc_id_slash_terminated coords =
        MakeQuadPres contents doc_id mode doc_id_slash_terminated coords

get_contents (MakeQuadPres contents doc_id mode doc_id_slash_terminated coords) = contents
get_doc_id (MakeQuadPres contents doc_id mode doc_id_slash_terminated coords) = doc_id
get_mode (MakeQuadPres contents doc_id mode doc_id_slash_terminated coords) = mode
get_doc_id_slash_terminated (MakeQuadPres contents doc_id mode doc_id_slash_terminated coords) = doc_id_slash_terminated
get_coords (MakeQuadPres contents doc_id mode doc_id_slash_terminated coords) = coords


type QuadPresType = QuadPres Contents_Node [String] Mode Bool QuadPresCoordsType

instance (Show a, Show b, Show c, Show d, Show e) => Show (QuadPres a b c d e) where
    show (MakeQuadPres contents doc_id mode doc_id_slash_terminated coords) =
        "QuadPres (doc_id = " ++ (show doc_id) ++ ", " ++
        "mode = " ++ (show mode) ++ ", " ++
        "doc_id_slash_terminated = " ++ (show doc_id_slash_terminated) ++ ", " ++
        "coords = " ++ (show coords) ++ ")"


church_N 0 f x = x
church_N n f x = (f (church_N (n-1) f x))


-- This function splits a string (or any list for that matter) according
-- to a static separator string.

--mysplit :: String -> String -> [String]
mysplit separator "" = [""]
mysplit separator base =
    let len = (length separator)
    in (if ((take len base) == separator)
        then "":(mysplit separator (church_N len tail base))
        else let ret = (mysplit separator (tail base))
             in (head(base):head(ret)) : tail(ret)
       )

remove_first_instances a [] = []
remove_first_instances a (b:bs) =
    if a == b
    then remove_first_instances a bs
    else b:bs

-- mysplit_perl behaves in a similar manner to perl's split in the sense
-- that it removes empty strings that are found at the end

mysplit_perl separator base = (reverse (remove_first_instances [] (reverse (mysplit separator base))))

--make_quadpres_instance :: Contents_Node -> String -> Mode -> QuadPresType
make_quadpres_instance contents doc_id_proto mode =
    (MakeQuadPres
        contents
        doc_id
        mode
        doc_id_slash_terminated
        coords
    ) where
        doc_id_slash_terminated :: Bool
        doc_id_slash_terminated = is_ending_with_slash doc_id_proto where
            is_ending_with_slash :: String -> Bool
            is_ending_with_slash [] = False
            is_ending_with_slash "/" = True
            is_ending_with_slash (a:as) = is_ending_with_slash as

        doc_id_parsed :: [String]
        doc_id_parsed = (mysplit_perl "/" doc_id_proto)

        doc_id = doc_id_parsed

        -- coords should contain
        -- coords = [Int]
        -- type = Image | Node
        -- mime = String
        coords :: QuadPresCoordsType
        coords = (find_coords contents ("":doc_id_parsed))



find_coords contents doc_id_parsed =
    (traverse [] contents []) where
    traverse coords branch old_path =
        ret where
            ret = (if path /= (take len_path doc_id_parsed)
                   then (MakeQuadPresCoords [] Node "")
                   else if (len_path == (length(doc_id_parsed)))
                   then (MakeQuadPresCoords coords Node "")
                   else if ((length matching_subs) > 0)
                        then (head matching_subs)
                        else if (len_path < ((length doc_id_parsed) - 1 ))
                             then (MakeQuadPresCoords [] Node "")
                             else (check_images 0 (get_images branch))
                  )
            ret2 = (path, (take len_path doc_id_parsed))
            path = old_path ++ [(get_url branch)]
            len_path = (length path)
            descend :: Int -> QuadPresCoordsType
            descend new_coord =
                (traverse
                    (coords ++ [new_coord])
                    ((get_subs branch)!!new_coord)
                    path
                )
            matching_subs =
                (filter
                    (\x -> ((length (get_coords_coords x)) > 0))
                   (map descend [0 .. ((length (get_subs branch))-1)])
                )
            last_element = (myle doc_id_parsed) where
                myle (a:[]) = a
                myle (a:as) = (myle as)
            check_images :: Int -> [Contents_Image String String] -> QuadPresCoordsType
            check_images n [] = (MakeQuadPresCoords [] Node "")
            check_images n images =
                (if last_element == (get_image_url (head images))
                 then (MakeQuadPresCoords (coords ++ [n]) Image (get_image_mime_type (head images)))
                 else check_images (n+1) (tail images))


result = make_quadpres_instance contents "intro.html" Server





